

// Name
//===================================
//
// Graphics
//


// DEPENDENCIES
//===================================

    #include "graphics.h"
    #include <SDL2/SDL_image.h>
    #include "unit.h"


// STATIC PROPERTIES
//===================================

// METHODS
//===================================

    Graphics::Graphics()
    {
        adjustMagnification();

        createWindow();
        createRenderer();
    };

    Graphics::~Graphics()
    {
        clearTextures();
        sdl2::SDL_Deleter( renderer_ );
        sdl2::SDL_Deleter( window_ );
    };

    const std::string Graphics::imgAddress( const std::string& relative_path ) const
    {
        const char* base_path = SDL_GetBasePath();

        if ( base_path == nullptr )
        {
            base_path = SDL_strdup("./");
        }

        const std::string address = base_path + relative_path;

        return address;
    };

    SDL_Texture* Graphics::loadTexture( const std::string& address ) const
    {
        SDL_RWops* rwop = SDL_RWFromFile( imgAddress( address ).c_str(), "rb" );

        if ( rwop == nullptr )
        {
            SDL_Log( "SDL_RWFromFile failed: %s", SDL_GetError() );
        }

        SDL_Surface* surface = IMG_LoadPNG_RW( rwop );
        SDL_Texture* texture = SDL_CreateTextureFromSurface( renderer_.get(), surface );

        SDL_FreeSurface( surface );

        if ( texture == nullptr )
        {
            SDL_Log( "SDL_CreateTextureFromSurface failed: %s", SDL_GetError() );
        }

        return texture;
    };

    bool Graphics::createRenderer()
    {
        if ( window_ == nullptr )
            return false;

        renderer_.reset( SDL_CreateRenderer( window_.get(), -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC ) );

        if ( renderer_ == nullptr )
        {
            SDL_Log( "SDL_CreateRenderer failed: %s", SDL_GetError() );
            return false;
        }

        return true;
    };

    bool Graphics::createWindow()
    {
        window_.reset( SDL_CreateWindow
        (
            "Boskeopolis Land",
            0,
            0,
            magnified( Unit::WINDOW_WIDTH_PIXELS ),
            magnified( Unit::WINDOW_HEIGHT_PIXELS ),
            //SDL_WINDOW_RESIZABLE
            SDL_WINDOW_FULLSCREEN_DESKTOP
        ) );

        if ( window_ == nullptr )
        {
            SDL_Log( "SDL_WindowCreation failed: %s", SDL_GetError() );
            return false;
        }

        return true;
    };

    void Graphics::clearTextures()
    {
        for ( auto& i : textures_ )
        {
            if ( i != nullptr )
            {
                //i.reset();
                sdl2::SDL_Deleter( i );
            }
        }
    };

    void Graphics::colorCanvas()
    {
        SDL_SetRenderDrawColor( renderer_.get(), 255, 255, 255, 255 );
        SDL_RenderFillRect( renderer_.get(), &screen_ );
    };

    void Graphics::clearScreen()
    {
        SDL_SetRenderDrawColor( renderer_.get(), 0, 0, 0, 255 );
        SDL_RenderClear( renderer_.get() );
    };

    void Graphics::presentScreen()
    {
        SDL_RenderPresent( renderer_.get() );
    };

    void Graphics::renderObject( SpriteSheet sheet, SDL_Rect source, SDL_Rect dest, SDL_RendererFlip flip )
    {
        if ( textures_[ sheet ] == nullptr )
        {
            textures_[ sheet ].reset( loadTexture( graphic_addresses_[ (int)sheet ] ) );
        }

        dest = sourceRelativeToScreen( dest );
        if ( SDL_RenderCopyEx( renderer_.get(), textures_[sheet].get(), &source, &dest, 0, 0, flip ) != 0 )
            printf( "Render failure: %s\n", SDL_GetError() );
    };

    int Graphics::magnified( int n ) const
    {
        return n * magnification_;
    };


    void Graphics::adjustMagnification()
    {
        SDL_Rect r;
        if ( SDL_GetDisplayBounds( 0, &r ) != 0 )
        {
            SDL_Log( "SDL_GetDisplayBounds failed: %s", SDL_GetError() );
            magnification_ = 1;
        }
        else
        {
            int monitor_width = r.w;
            int monitor_height = r.h;

            double screen_aspect_ratio = (double)Unit::WINDOW_WIDTH_PIXELS / (double)Unit::WINDOW_HEIGHT_PIXELS;
            double monitor_aspect_ratio = (double)monitor_width / (double)monitor_height;

            if ( monitor_aspect_ratio > screen_aspect_ratio )
            {
                magnification_ = floor( monitor_height / (double)Unit::WINDOW_HEIGHT_PIXELS );
            }
            else
            {
                magnification_ = floor( monitor_width / (double)Unit::WINDOW_WIDTH_PIXELS );
            }

            left_edge_ = ( monitor_width - magnified( Unit::WINDOW_WIDTH_PIXELS ) ) / 2;
            top_edge_ = ( monitor_height - magnified( Unit::WINDOW_HEIGHT_PIXELS ) ) / 2;

            adjustScreen();
        }
    };

    void Graphics::adjustScreen()
    {
        screen_.x = left_edge_;
        screen_.y = top_edge_;
        screen_.w = magnified( Unit::WINDOW_WIDTH_PIXELS );
        screen_.h = magnified( Unit::WINDOW_HEIGHT_PIXELS );
    };

    SDL_Rect Graphics::sourceRelativeToScreen( SDL_Rect source ) const
    {
        return
        {
            magnified( source.x ) + left_edge_,
            source.y = magnified( source.y ) + top_edge_,
            source.w = magnified( source.w ),
            source.h = magnified( source.h ),
        };
    };
