
// Name
//===================================
//
// Sprite
//

#ifndef SPRITE_H
#define SPRITE_H


// FORWARD DECLARATIONS
//===================================

    class Block;
    class Camera;
    class Collision;
    class EventSystem;
    class Input;
    class Inventory;
    class Map;
    class SpriteGraphics;
    class SpriteMovement;


// DEPENDENCIES
//===================================

    #include "direction.h"
    #include "graphics.h"
    #include "object.h"
    #include <SDL2/SDL.h>
    #include "unit.h"


// CLASS
//===================================

    class Sprite : public Object
    {
        public:
            enum Type
            {
                HERO,
                BLOCK,
                ENEMY,
                FLAT_TOP,
                LADDER,
                NOTYPE
            };

            Sprite
            (
                SpriteGraphics* graphics = nullptr,
                SpriteMovement* movement = nullptr,
                int x = 0,
                int y = 0,
                int width = 16,
                int height = 16,
                Type type = Type::NOTYPE,
                int texture_x = 27 * 8,
                int texture_y = 8
            );
            ~Sprite();

            bool jump_lock_ = false;

            void update( Input& input, Camera& camera, Map& lvmap );
            void render( Graphics& graphics, Camera& camera );

            bool onGround() const;
            bool onGroundPrev() const;
            bool canJump() const;
            bool isJumping() const;
            bool isRunning() const;
            bool isDucking() const;
            bool isMoving() const;
            bool lookingUp() const;

            bool onLadder() const;
            bool onLadderPrev() const;
            bool touchingLadder() const;
            bool touchingLadderPrev() const;

            void moveLeft();
            void moveRight();
            void stop();
            void stopDucking();
            SDL_Rect adjustBoundBoxes();
            void collideStopXLeft( int overlap );
            void collideStopXRight( int overlap );
            void collideStopYBottom( int overlap );
            void collideStopYTop( int overlap );
            void jump();
            void run();
            void duck();
            void unduck();
            void slowFall();
            void stopY();
            void lookUp();
            void stopLookingUp();
            void kill();
            void boundaries( Camera& camera );

            void touchLadder();
            void grabLadder();
            void releaseLadder();
            void moveUp();
            void moveDown();

            void collisionDetect( Sprite& other );
            void collisionDetect( Block& block, Map& lvmap, EventSystem& events, Inventory& inventory );

            Direction::Horizontal directionX() const;
            Type type() const;

            void debugResetPos();

        private:
            const Type type_ = Type::NOTYPE;

            const int START_SPEED_WALK = 160;
            const int TOP_SPEED_WALK = 2000;
            const int START_SPEED_RUN = 160 * 2;
            const int TOP_SPEED_RUN = 2000 * 2;

            const int GRAVITY_START_SPEED = 500;
            const int GRAVITY_TOP_SPEED = 4000;
            const int JUMP_START_SPEED = 1500;
            const int JUMP_TOP_SPEED = 7200;
            const int JUMP_DUCK_TOP_SPEED = 5500;

            const double BOUNCE = .2;

            const Graphics::SpriteSheet texture_ = Graphics::SpriteSheet::LVSPRITES_AUTUMN;
            int texture_x_ = 0;
            int texture_y_ = 0;
            SpriteMovement* movement_ = nullptr;
            SpriteGraphics* graphics_ = nullptr;

            int vx_ = 0;
            int vy_ = 0;
            int acceleration_x_ = 0;
            int acceleration_y_ = 0;
            int gravity_start_speed_ = GRAVITY_START_SPEED;
            int gravity_top_speed_ = GRAVITY_TOP_SPEED;
            int start_speed_ = START_SPEED_WALK;
            int top_speed_ = TOP_SPEED_WALK;
            int jump_top_speed_ = JUMP_TOP_SPEED;
            Direction::Horizontal direction_x_ = Direction::Horizontal::LEFT;

            void startJump();

            bool is_running_ = false;
            bool on_ground_ = false;
            bool on_ground_prev_ = false;
            bool can_jump_ = false;
            bool is_jumping_ = false;
            bool jump_start_ = false;
            bool jump_end_ = false;
            bool is_ducking_ = false;
            bool is_moving_ = false;
            bool on_ladder_ = false;
            bool on_ladder_prev_ = false;
            bool touching_ladder_ = false;
            bool touching_ladder_prev_ = false;
            bool looking_up_ = false;
            bool dead_ = false;

            void position();
    };

#endif // SPRITE_H

